import os
import commands
import ujson


from edge import eventlog
from edge.com import fs, net
from edge.exceptions import *
from edge.types import FormatTemplate
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg

import edge_creator as creator

UI_NETWORK_JNODE_FILEPATH = '/etc/edge/setupstore/admin/actions/network.json'
MANAGEMENT_MAC_FILEPATH = '/etc/network/management-mac'


# noinspection PyClassHasNoInit
class RtspStreamingApi:
    @classmethod
    def load_unit(cls, uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_RTSP_STREAMING)

        uiconf = uimod.load_conf(default=None)
        if not uiconf:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        ph.CamDgStats.create(uimod.pid).test_availability()

        uiconf_pub = uiconf['rw']
        try:
            rtsp_url = cls._make_rtsp_url(uiconf_pub)
        except (ValueError, RuntimeError, IOError):
            eventlog.log_e('fail: InternalError')
            raise InternalError

        uiconf_pub['streaming']['url'] = rtsp_url
        return {
            'type': ph.UI_TYPE_RTSP_STREAMING,
            'conf': uiconf_pub
        }

    @classmethod
    def save_unit(cls, uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        :rtype: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_RTSP_STREAMING)

        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        ph.CamDgStats.create(uimod.pid).test_availability()

        uiconf_pub = uiconf['rw']

        def validate_uimod_pub():
            fs.validate_json(new_uiconf_conf, uimod.schema_path)

            new_streaming = new_uiconf_conf['streaming']
            if new_streaming['transport'] == 'rtp-over-udp-or-tcp':
                if new_streaming['min-udp-port'] == 0 and new_streaming['max-udp-port'] != 0:
                    eventlog.log_d('fail: ParamsError')
                    raise ParamsError

                if new_streaming['max-udp-port'] == 0 and new_streaming['min-udp-port'] != 0:
                    eventlog.log_d('fail: ParamsError')
                    raise ParamsError

            # validate 'format' jnodes from json
            footer_rows = new_uiconf_conf['live']['footer']['rows']
            if cls._validate_format(footer_rows) is False:
                eventlog.log_d('fail: ParamsError')
                raise ParamsError

            old_rtsp_url = cls._make_rtsp_url(uiconf_pub)
            new_rtsp_url = new_uiconf_conf['streaming']['url']
            if old_rtsp_url != new_rtsp_url:
                eventlog.log_d('fail: ParamsError (rtsp-url is read-only)')
                raise ParamsError

        validate_uimod_pub()

        new_uiconf_conf['streaming']['url'] = cls._make_rtsp_url(new_uiconf_conf)

        if uiconf_pub == new_uiconf_conf:
            return new_uiconf_conf

        restart_setupstore = []

        if uiconf['rw']['enabled'] != new_uiconf_conf['enabled']:
            restart_setupstore.append(True)

        uiconf['rw'] = new_uiconf_conf
        uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
        creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})

        if restart_setupstore:
            dg.LifeCycle.sighup_ss_dgmod()

        return {
            'type': ph.UI_TYPE_RTSP_STREAMING,
            'conf': new_uiconf_conf
        }

    @staticmethod
    def _make_rtsp_url(ui_rw_jnode):
        """
        :type ui_rw_jnode: dict
        :rtype: str
        """

        try:
            with open(UI_NETWORK_JNODE_FILEPATH) as json_file:
                ui_network_jnode = ujson.load(json_file)
            mac_address = ui_network_jnode['ro']['mac']

        except IOError as x:
            if x.errno == 2:
                if os.path.exists(MANAGEMENT_MAC_FILEPATH):
                    with open(MANAGEMENT_MAC_FILEPATH, 'r') as mgmt_mac_file:
                        mac_address = mgmt_mac_file.readlines()[0].strip()
                else:
                    raise

            else:
                eventlog.log_d('fail: read network settings')
                raise IOError

        ifname = net.get_ifname(mac_address)  # TODO: check if ifname is not None
        try:
            ip_addr = commands.getoutput("ip addr show %s | grep -v '127.0.0.1' | grep -w inet | awk '{print $2}'"
                                         % ifname).split('/')[0]
        except Exception:
            eventlog.log_d('fail: discover ip-address')
            raise RuntimeError

        tcp_port = str(ui_rw_jnode['streaming']['tcp-port'])
        ui_rtsp_codec = ui_rw_jnode['live']['videostream']['format']
        if ui_rtsp_codec == 'mjpeg':
            rtsp_codec = 'mjpeg'
        elif ui_rtsp_codec == 'h264-cqp' or 'h264-cbr':
            rtsp_codec = 'h264'
        else:
            eventlog.log_d('unknown rtsp-codec %s' % ui_rtsp_codec)
            raise ValueError

        rtsp_url = 'rtsp://' + ip_addr + ':' + tcp_port + '/live-' + rtsp_codec

        return rtsp_url

    @staticmethod
    def _validate_format(jnode):
        """
        :type jnode: list
        :rtype: bool
        """
        key_to_true_map = {
            'datetime': True,
            'datetime-ms': True,
            'post-name': True,
            'geolocation': True,
            'vendor': True,
            'model': True,
            'serial': True
        }

        for row in jnode:
            for col in row['cols']:
                try:
                    FormatTemplate(template=col['format']).substitute(key_to_true_map)
                except KeyError as x:
                    eventlog.log_d('fail: invalid key (%s)' % x.message)
                    return False

        return True
